﻿using System;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Appearance = Infragistics.Win.Appearance;

namespace ConversationControl
{
    //TODO 
    //Correct cell padding issue - without the newline in the "FormatText" string, the text gets cut off at the bottom of the cell.  Even with this in place, text is cut off on long text strings.
    //Make the cells be non-selectable
    //Make the first cell not be selected automatically (this turns its text white)
    //Multiline text support


    public partial class ConversationControl : UserControl
    {
        private const string FormatText = "<p><span style=\"float:left;clear:left;font-weight:bold;\">{0}</span> <span style=\"float:right;font-weight:bold;\">{1}</span></p>  <p><span style=\"float:left;clear:left;\">{2}</span></p>";//<p><br></br></p>";
        private readonly DataTable _dataTable = new DataTable();

        public enum MessageAlignment
        {
            Left,
            Right
        }

        public void AddMessage(string user, DateTime time, string message, Color foreColor, Color backColor, MessageAlignment messageAlignment)
        {
            _dataTable.Rows.Add(null, user, time, message, foreColor, backColor, messageAlignment);

            //Move the latest message to the bottom of the grid
            var row = ultraGrid1.Rows[ultraGrid1.Rows.Count - 1];
            ultraGrid1.ActiveRowScrollRegion.ScrollRowIntoView(row);
        }

        public void Clear()
        {
            _dataTable.Clear();
        }

        public ConversationControl()
        {
            InitializeComponent();
            HookupDrawFilter();
            SetupGridData();

            //TODO - Figure out how to get the first cell to not be selected or active or whatever.  Also, figure out how to not have any cells active.
            //ultraGrid1.ActiveRow = null;
            
            //foreach (var cell in ultraGrid1.Selected.Cells)
            //{
            //    cell.Selected = false;
            //}
        }

        private void HookupDrawFilter()
        {
            var myCreation = new CellCreationFilter { VOffset = 50 };
            ultraGrid1.CreationFilter = myCreation;
            var myDraw = new CellDrawFilter { CornerRadius = 10 };
            ultraGrid1.DrawFilter = myDraw;
        }

        private void SetupGridData()
        {          
            _dataTable.Columns.Add("id", typeof (int)).AutoIncrement = true;
            _dataTable.Columns.Add("Name", typeof (string));
            _dataTable.Columns.Add("Date", typeof(DateTime));
            _dataTable.Columns.Add("Note", typeof (string));
            _dataTable.Columns.Add("ForeColor", typeof (Color));
            _dataTable.Columns.Add("BackColor", typeof (Color));
            _dataTable.Columns.Add("MessageAlignment", typeof (MessageAlignment));

            ultraGrid1.DataSource = _dataTable;
        }

        private void UltraGrid1InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            if (e.Layout.Bands[0] != null)
            {
                e.Layout.Bands[0].Columns[0].Hidden = true;
                e.Layout.Bands[0].Override.RowSelectorHeaderStyle = RowSelectorHeaderStyle.None;
                e.Layout.Bands[0].Columns[1].Hidden = true;
                e.Layout.Bands[0].Columns[2].Hidden = true;
                e.Layout.Bands[0].Columns[3].Hidden = true;
                e.Layout.Bands[0].Columns[4].Hidden = true;
                e.Layout.Bands[0].Columns[5].Hidden = true;
                e.Layout.Bands[0].Columns[6].Hidden = true;
                e.Layout.Bands[0].ColHeadersVisible = false;
                e.Layout.Bands[0].Override.CellPadding = 10;
                e.Layout.Bands[0].Override.CellSpacing = 3;
                e.Layout.Override.ActiveAppearancesEnabled = Infragistics.Win.DefaultableBoolean.False;
                e.Layout.Override.SelectedAppearancesEnabled = Infragistics.Win.DefaultableBoolean.False;
                UltraGridColumn col = e.Layout.Bands[0].Columns.Add("Conversation");
                col.Width = 270;
                col.CellMultiLine = Infragistics.Win.DefaultableBoolean.True;
                col.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.FormattedText;
                e.Layout.Override.RowSizing = RowSizing.AutoFree;

                //ultraGrid1.DisplayLayout.AutoFitStyle = AutoFitStyle.ResizeAllColumns;
            }
        }

        private void UltraGrid1InitializeRow(object sender, InitializeRowEventArgs e)
        {
            var appearance = new Appearance
                                 {
                                     BackColor = (Color) e.Row.Cells["BackColor"].Value,
                                     ForeColor = (Color) e.Row.Cells["ForeColor"].Value
                                 };

            //Add right-aligned messages to that list in the filter
            if ((MessageAlignment)e.Row.Cells["MessageAlignment"].Value == MessageAlignment.Right)
            {
                ((CellCreationFilter)ultraGrid1.CreationFilter).RightAlignedCellIDs.Add((int)e.Row.Cells["id"].Value);
            }

            e.Row.Cells["Conversation"].Appearance = appearance;
            e.Row.Cells["Conversation"].SetValue(string.Format(FormatText, e.Row.Cells["Name"].Value,e.Row.Cells["Date"].Value, e.Row.Cells["Note"].Value), false);         
        }

        private void ultraGrid1_BeforeRowActivate(object sender, RowEventArgs e)
        {
           
        }

        private void ultraGrid1_AfterRowActivate(object sender, EventArgs e)
        {
            ultraGrid1.ActiveRow = null;
        } 
  
    }
}


  
 