﻿using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using System.Diagnostics;

namespace ConversationControl
{
    class CellCreationFilter : IUIElementCreationFilter
    {

        private int _voffset = 10;
        
        List<int> _rightAlignedCellIDs = new List<int>();   
 
        public int VOffset
        {
            get { return _voffset; }
            set { _voffset = value; }
        }

        

        public List<int> RightAlignedCellIDs
        {
            get { return _rightAlignedCellIDs; }
            set { _rightAlignedCellIDs = value; }
        }

        #region IUIElementCreationFilter Members

        public void AfterCreateChildElements(UIElement parent)
        {

            UltraGridUIElement grid = parent as UltraGridUIElement;
            if (grid != null)
            {
                UltraGrid grd = (UltraGrid)grid.Control;
                grd.DisplayLayout.Bands[0].Columns["Conversation"].Width = grid.Rect.Width - _voffset - 22;
            }

            
            var cell = parent as CellUIElement;
            if (cell != null)
            {
                
                if (_rightAlignedCellIDs.Contains((int)cell.Row.Cells["id"].Value))
                {
                    cell.Rect = new Rectangle(new Point(cell.Parent.Rect.X + _voffset, cell.Parent.Rect.Y + 1),
                       new Size(cell.Rect.Width, cell.Rect.Height));
                }
                else
                {
                    cell.Rect = new Rectangle(new Point(cell.Rect.Location.X, cell.Parent.Rect.Y + 1),
                        new Size(cell.Rect.Width, cell.Rect.Height));
                }
                
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        #endregion
    }

    class CellDrawFilter : IUIElementDrawFilter
    {
        private int _cornerRadius = 3;

        public int CornerRadius
        {
            get { return _cornerRadius; }
            set { _cornerRadius = value; }
        }


        #region IUIElementDrawFilter Members

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            if (drawParams.Element is RowUIElement)
            {
                drawParams.AppearanceData.BorderAlpha = Alpha.Transparent;
                Debug.WriteLine(drawPhase);
                return true;
            }

           

            if (drawParams.Element is CellUIElement)// && drawPhase == DrawPhase.AfterDrawElement)
            {

                int xPosition = drawParams.Element.Rect.X;
                int yPosition = drawParams.Element.Rect.Y;
                int width = drawParams.Element.Rect.Width;
                int height = drawParams.Element.Rect.Height;


                using (var path = new GraphicsPath())
                {
                    path.AddLine(xPosition + _cornerRadius, yPosition, xPosition + width - (_cornerRadius * 2), yPosition);
                    path.AddArc(xPosition + width - (_cornerRadius * 2), yPosition, _cornerRadius * 2, _cornerRadius * 2, 270, 90);
                    path.AddLine(xPosition + width, yPosition + _cornerRadius, xPosition + width, yPosition + height - (_cornerRadius * 2));
                    path.AddArc(xPosition + width - (_cornerRadius * 2), yPosition + height - (_cornerRadius * 2), _cornerRadius * 2, _cornerRadius * 2, 0, 90);
                    path.AddLine(xPosition + width - (_cornerRadius * 2), yPosition + height, xPosition + _cornerRadius, yPosition + height);
                    path.AddArc(xPosition, yPosition + height - (_cornerRadius * 2), _cornerRadius * 2, _cornerRadius * 2, 90, 90);
                    path.AddLine(xPosition, yPosition + height - (_cornerRadius * 2), xPosition, yPosition + _cornerRadius);
                    path.AddArc(xPosition, yPosition, _cornerRadius * 2, _cornerRadius * 2, 180, 90);
                    path.CloseFigure();

                    var boxPen = new Pen(Color.Black, 3);
                    drawParams.Graphics.DrawPath(boxPen, path);
                    var b = new SolidBrush(((CellUIElement)drawParams.Element).Cell.Appearance.BackColor);
                    drawParams.Graphics.FillRegion(b, new Region(path));
                    b.Dispose();
                    boxPen.Dispose();
                }
                if (drawPhase == DrawPhase.BeforeDrawBackColor)
                {
                    drawParams.AppearanceData.BackColorAlpha = Alpha.Transparent;
                    return true;
                }
                
            }
            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            if (drawParams.Element is RowUIElement )
                return  DrawPhase.BeforeDrawBorders| DrawPhase.BeforeDrawFocus;
            
            if (drawParams.Element is CellUIElement)
                return DrawPhase.BeforeDrawElement | DrawPhase.BeforeDrawBackColor ;
            

            return DrawPhase.None;
        }

        #endregion
    }
}
